<?php
$user_id = '1';
$request_type = 'payment_v2'; // payment_v2, withdraw_v2, payment_query_v2, withdraw_query_v2, balance_v2
$base_url = 'http://service.local.test';
$safe_code = '8b998abba49ae33a194cb9677b13ecbe';
$guid = 'smajhv5r2etr';
$transaction_signature = 'f55c19747fde060a4af40dc6ed187ae3';

$routes = [
    'payment_v2' => "/merchant/v2/$guid/payment",
    'withdraw_v2' => "/merchant/v2/$guid/withdraw",
    'payment_query_v2' => "/merchant/v2/$guid/order/payment",
    'withdraw_query_v2' => "/merchant/v2/$guid/order/withdraw",
    'balance_v2' => "/merchant/v2/$guid/balance",
];

$params = [
    'user_id' => $user_id,
    'order_id' => 'TEST'.$user_id.time(),
    'amount' => rand(10, 1000),
    'currency' => 'PHP',
    'channel' => 'QRPH',
    'callback_url' => 'http://payment.local.test/callback',
    'redirect_url' => 'http://payment.local.test/redirect',
    'remark' => 'test',
    'timestamp' => time(),
    'email' => 'test@dragonpay.ph',
    'phone' => '09175123213',
];

try {
    $response = send_request($request_type, $params);

    echo json_encode($response);
    if ($response['code'] !== '1000') {
        throw new Exception($response['message']);
    }
} catch (Exception $e) {
    echo $e->getMessage();
}

function send_request(string $request_type, array $params): array
{
    global $safe_code, $base_url, $routes, $transaction_signature;

    $params['sign'] = generate_signature($params, $safe_code, $request_type);
    $json_string = json_encode($params);
    $encrypt = openssl_encrypt($json_string, 'AES-256-ECB', $safe_code, OPENSSL_RAW_DATA);
    $params = base64_encode($encrypt);

    $transaction_signature = transaction_signature($params, $transaction_signature);
    $header = [
        'Content-Type: text/plain',
        'X-Transaction-Signature: '.$transaction_signature
    ];

    $response = send_curl_request($base_url.$routes[$request_type], $params, $header);

    return json_decode($response, true);
}

function send_curl_request($url, $body, $header): string
{
    $curl = curl_init();
    curl_setopt_array($curl, [
        CURLOPT_URL => $url,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => 'utf-8',
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => 'POST',
        CURLOPT_POSTFIELDS => $body,
        CURLOPT_HTTPHEADER => $header,
        CURLOPT_USERAGENT => 'Mozilla/5.0 (Windows NT 6.1; WOW64; rv:38.0) Gecko/20100101 Firefox/38.0',
    ]);
    $response = curl_exec($curl);
    curl_close($curl);

    return $response;
}

function generate_signature($params, $safe_code, $sign_type): string
{
    $serialize_sign = serialize_sign($params, $safe_code, $sign_type);
    $private_key = openssl_get_privatekey(file_get_contents('private_key.pem'));
    openssl_sign($serialize_sign, $signature, $private_key, 'SHA256');
    openssl_free_key($private_key);

    return base64_encode($signature);
}

function serialize_sign(array $params, string $safe_code, string $sign_type): string
{
    switch ($sign_type) {
        case 'payment_v2':
        case 'withdraw_v2':
            $sign_fields = [
                'user_id', 'order_id', 'amount', 'currency', 'channel',
                'timestamp',
            ];
            break;
        case 'payment_query_v2':
        case 'withdraw_query_v2':
            $sign_fields = ['user_id', 'order_id',  'timestamp'];
            break;
        case 'balance_v2':
            $sign_fields = ['user_id', 'timestamp'];
            break;

        case 'payment_response':
            $sign_fields = [
                'user_id', 'order_id', 'transaction_id', 'channel',
                'submit_currency', 'submit_amount', 'accept_currency', 'accept_amount',
                'exchange_rate', 'pay_url',
            ];
            break;
        case 'withdraw_response':
            $sign_fields = [
                'user_id', 'order_id', 'transaction_id', 'channel',
                'submit_currency', 'submit_amount', 'accept_currency', 'accept_amount',
                'exchange_rate',
            ];
            break;
        case 'payment_query_response':
        case 'withdraw_query_response':
            $sign_fields = [
                'user_id', 'order_id', 'transaction_id', 'channel',
                'submit_currency', 'submit_amount', 'accept_currency', 'accept_amount',
                'exchange_rate', 'status', 'timestamp',
            ];
            break;
        case 'balance_response':
        default:
            $sign_fields = ['user_id'];
            break;
    }

    if ($sign_fields) {
        $params = array_intersect_key($params, array_flip($sign_fields));
    }

    ksort($params);

    $sign = '';
    foreach ($params as $key => $value) {
        $sign .= $key.'='.$value.'&';
    }
    $sign .= $safe_code;

    return $sign;
}

function verify_signature($response, $safe_code, $signType)
{
    $params = $response['data'];
    $signature = base64_decode($params['sign']);

    $serialize_sign = serialize_sign($params, $safe_code, $signType);

    $pubKey = openssl_get_publickey(file_get_contents('plateform_public_key.pem'));
    $verified = openssl_verify($serialize_sign, $signature, $pubKey, OPENSSL_ALGO_SHA256);
    openssl_free_key($pubKey);

    return $verified !== 0;
}

function transaction_signature(string $requestBody, string $transactionSecret): string
{
    return base64_encode(hash_hmac('sha256', $requestBody, $transactionSecret, true));
}
exit();